////////////////////////////////////////////////////////////////////////////////
/** @brief I2Cで周辺のICと通信するための関数
@attention タイムアウト時間が1msになっているので、DMAで30個以上データを送受信するような場合はタイムアウト時間を引き延ばす必要あり
           ただし、仕様上60ms程度が限度である
           STM32のDMAの仕様上、タイムアウトしてもソフトからDMAを停止することはできず、
           DMA転送中のタイムアウトは検出できるが、通信をリセットすることはできない
@date 2012/9/27
*///////////////////////////////////////////////////////////////////////////////
#include <stm32f4xx.h>
#include "config_I2C.h"

////////////////////////////////////////////////////////////////////////////////
/*
使用するペリフェラル
- I2C1
- I2C2
- I2C3

- TIM2

- DMA1-6-1(I2C1TX)
- DMA1-5-1(I2C1RX)

- DMA1-7-7(I2C2TX)
- DMA1-3-7(I2C2RX)

- DMA1-4-3(I2C3TX)
- DMA1-2-3(I2C3RX)

使用するポート

          SCL       SDA
- I2C1  PB6orPB8  PB7orPB9
- I2C2    PB10      PB11
- I2C3    PA8       PC9

*///////////////////////////////////////////////////////////////////////////////

//I2C1

#define RCC_ClockCmd_I2Cx(x)        RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, x)
#define RCC_ClockCmd_GPIOx_SCL(x)   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, x)
#define RCC_ClockCmd_GPIOx_SDA(x)   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, x)
#define RCC_ClockCmd_DMAx(x)        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA1, x)
#define I2Cx_SIO                    I2C1
#define GPIOx_I2Cx_SCL              GPIOB
#define GPIOx_I2Cx_SDA              GPIOB
#define GPIO_AF_I2Cx_SIO            GPIO_AF_I2C1
#define GPIO_Pin_x_I2C_SCL          GPIO_Pin_8
#define GPIO_Pin_x_I2C_SDA          GPIO_Pin_9
#define GPIO_PinSourceX_I2C_SCL     GPIO_PinSource8
#define GPIO_PinSourceX_I2C_SDA     GPIO_PinSource9
#define DMAx_StreamY_I2C_TX         DMA1_Stream6
#define DMAx_StreamY_I2C_RX         DMA1_Stream5
#define DMA_Channel_x_I2C_TX        DMA_Channel_1
#define DMA_Channel_x_I2C_RX        DMA_Channel_1
#define DMA_FLAG_TCIFx_TX           DMA_FLAG_TCIF6 // Streamの番号と同じ
#define DMA_FLAG_TCIFx_RX           DMA_FLAG_TCIF5 // Streamの番号と同じ
#define DMAx_StreamY_IRQn_I2C_RX    DMA1_Stream5_IRQn
#define I2C_DMAx_StreamY_IRQHandler DMA1_Stream5_IRQHandler
#define DMA_IT_TCIFx_RX             DMA_IT_TCIF5 // Streamの番号と同じ
#define I2C_CLK                     400000 // SCLの周波数[Hz]

//I2C2
/*
#define RCC_ClockCmd_I2Cx(x)        RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, x)
#define RCC_ClockCmd_GPIOx_SCL(x)   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, x)
#define RCC_ClockCmd_GPIOx_SDA(x)   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, x)
#define RCC_ClockCmd_DMAx(x)        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA1, x)
#define I2Cx_SIO                    I2C2
#define GPIOx_I2Cx_SCL              GPIOB
#define GPIOx_I2Cx_SDA              GPIOB
#define GPIO_AF_I2Cx_SIO            GPIO_AF_I2C2
#define GPIO_Pin_x_I2C_SCL          GPIO_Pin_10
#define GPIO_Pin_x_I2C_SDA          GPIO_Pin_11
#define GPIO_PinSourceX_I2C_SCL     GPIO_PinSource10
#define GPIO_PinSourceX_I2C_SDA     GPIO_PinSource11
#define DMAx_StreamY_I2C_TX         DMA1_Stream7
#define DMAx_StreamY_I2C_RX         DMA1_Stream3
#define DMA_Channel_x_I2C_TX        DMA_Channel_7
#define DMA_Channel_x_I2C_RX        DMA_Channel_7
#define DMA_FLAG_TCIFx_TX           DMA_FLAG_TCIF7 // Streamの番号と同じ
#define DMA_FLAG_TCIFx_RX           DMA_FLAG_TCIF3 // Streamの番号と同じ
#define DMAx_StreamY_IRQn_I2C_RX    DMA1_Stream3_IRQn
#define I2C_DMAx_StreamY_IRQHandler DMA1_Stream3_IRQHandler
#define DMA_IT_TCIFx_RX             DMA_IT_TCIF3 // Streamの番号と同じ
#define I2C_CLK                     400000 // SCLの周波数[Hz]
*/
//I2C3
/*
#define RCC_ClockCmd_I2Cx(x)        RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C3, x)
#define RCC_ClockCmd_GPIOx_SCL(x)   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, x)
#define RCC_ClockCmd_GPIOx_SDA(x)   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, x)
#define RCC_ClockCmd_DMAx(x)        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA1, x)
#define I2Cx_SIO                    I2C3
#define GPIOx_I2Cx_SCL              GPIOA
#define GPIOx_I2Cx_SDA              GPIOC
#define GPIO_AF_I2Cx_SIO            GPIO_AF_I2C3
#define GPIO_Pin_x_I2C_SCL          GPIO_Pin_8
#define GPIO_Pin_x_I2C_SDA          GPIO_Pin_9
#define GPIO_PinSourceX_I2C_SCL     GPIO_PinSource8
#define GPIO_PinSourceX_I2C_SDA     GPIO_PinSource9
#define DMAx_StreamY_I2C_TX         DMA1_Stream4
#define DMAx_StreamY_I2C_RX         DMA1_Stream2
#define DMA_Channel_x_I2C_TX        DMA_Channel_3
#define DMA_Channel_x_I2C_RX        DMA_Channel_3
#define DMA_FLAG_TCIFx_TX           DMA_FLAG_TCIF4 // Streamの番号と同じ
#define DMA_FLAG_TCIFx_RX           DMA_FLAG_TCIF2 // Streamの番号と同じ
#define DMAx_StreamY_IRQn_I2C_RX    DMA1_Stream2_IRQn
#define I2C_DMAx_StreamY_IRQHandler DMA1_Stream2_IRQHandler
#define DMA_IT_TCIFx_RX             DMA_IT_TCIF2 // Streamの番号と同じ
#define I2C_CLK                     400000 // SCLの周波数[Hz]
*/
#define RCC_ClockCmd_I2C_TIM(x)     RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, x)
#define TIMx_I2C_TIM                TIM2
#define PCLKx_Frequency_I2C_TIM     PCLK1_Frequency
#define TIMEOUT                     0.001 // タイムアウト時間[s]、仕様上60ms程度が限度

#define MPU_6050_SLAVE_ADDRESS      0b1101000
#define AK_8975_SLAVE_ADDRESS       0X0C

#define I2C_RX_BUFFER_SIZE          (6*3+14)

// 作業用変数
volatile uint8_t ADget_mode = 0;
volatile uint8_t Stop_IT_flag = 0; // このフラグをセットして割り込みを停止する

// 受信データ
uint8_t RX_data[I2C_RX_BUFFER_SIZE] = {0}; // こちらはRX_data[0]から値が書きこまれる

// AD変換値
int16_t AD_data_axel_x;
int16_t AD_data_axel_y;
int16_t AD_data_axel_z;
int16_t AD_data_temp;
int16_t AD_data_gyro_x[4];
int16_t AD_data_gyro_y[4];
int16_t AD_data_gyro_z[4];

DMA_InitTypeDef DMA_InitStructure_I2C_TX;
DMA_InitTypeDef DMA_InitStructure_I2C_RX;

////////////////////////////////////////////////////////////////////////////////
/** @brief センサと通信するためにI2Cを設定する
@details
基本的にマスターとして使うことを想定しており、スレーブ用の設定はしていない
以下にある1バイト送受信関数、DMAを用いた多バイト送信関数ともにこの初期化関数で使用できるようになる
@attention
使用するI2Cペリフェラル, ポートはマクロで定義すること
@return なし
*///////////////////////////////////////////////////////////////////////////////
void I2C_DMA_config_for_communication_with_MPU9150(void)
{
	// クロックを有効にする
	RCC_ClockCmd_I2Cx(ENABLE);
	RCC_ClockCmd_GPIOx_SCL(ENABLE);
	RCC_ClockCmd_GPIOx_SDA(ENABLE);
	RCC_ClockCmd_DMAx(ENABLE);

	// 入出力ポートにペリフェラルを接続する
	GPIO_PinAFConfig(GPIOx_I2Cx_SCL, GPIO_PinSourceX_I2C_SCL , GPIO_AF_I2Cx_SIO);
	GPIO_PinAFConfig(GPIOx_I2Cx_SDA, GPIO_PinSourceX_I2C_SDA , GPIO_AF_I2Cx_SIO);

	// I2C通信用ポートの設定

	// SCL
	GPIO_InitTypeDef  GPIO_InitStructure;
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_x_I2C_SCL;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOx_I2Cx_SCL, &GPIO_InitStructure);

	// SDA
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_x_I2C_SDA;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOx_I2Cx_SDA, &GPIO_InitStructure);

	// I2Cの設定
	I2C_InitTypeDef I2C_InitStructure;
	I2C_StructInit(&I2C_InitStructure);
	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStructure.I2C_ClockSpeed = I2C_CLK;
	I2C_Init(I2Cx_SIO, &I2C_InitStructure);

	// I2Cデータ送信用DMACの設定
	DMA_StructInit(&DMA_InitStructure_I2C_TX);
	DMA_InitStructure_I2C_TX.DMA_Channel = DMA_Channel_x_I2C_TX;
	DMA_InitStructure_I2C_TX.DMA_PeripheralBaseAddr = (uint32_t)(&(I2Cx_SIO->DR));
//	DMA_InitStructure_I2C_TX.DMA_Memory0BaseAddr = (uint32_t)TX_data;
	DMA_InitStructure_I2C_TX.DMA_DIR = DMA_DIR_MemoryToPeripheral;
	DMA_InitStructure_I2C_TX.DMA_BufferSize = 0; // 転送回数
	DMA_InitStructure_I2C_TX.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure_I2C_TX.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure_I2C_TX.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure_I2C_TX.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure_I2C_TX.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStructure_I2C_TX.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure_I2C_TX.DMA_FIFOMode = DMA_FIFOMode_Disable;
	DMA_InitStructure_I2C_TX.DMA_MemoryBurst = DMA_MemoryBurst_Single;
	DMA_InitStructure_I2C_TX.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
	DMA_Init(DMAx_StreamY_I2C_TX, &DMA_InitStructure_I2C_TX);

	// I2Cデータ受信用DMACの設定
	DMA_StructInit(&DMA_InitStructure_I2C_RX);
	DMA_InitStructure_I2C_RX.DMA_Channel = DMA_Channel_x_I2C_RX;
	DMA_InitStructure_I2C_RX.DMA_PeripheralBaseAddr = (uint32_t)(&(I2Cx_SIO->DR));
	DMA_InitStructure_I2C_RX.DMA_Memory0BaseAddr = (uint32_t)RX_data;
	DMA_InitStructure_I2C_RX.DMA_DIR = DMA_DIR_PeripheralToMemory;
	DMA_InitStructure_I2C_RX.DMA_BufferSize = 0; // 転送回数
	DMA_InitStructure_I2C_RX.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure_I2C_RX.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure_I2C_RX.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure_I2C_RX.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure_I2C_RX.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStructure_I2C_RX.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure_I2C_RX.DMA_FIFOMode = DMA_FIFOMode_Disable;
	DMA_InitStructure_I2C_RX.DMA_MemoryBurst = DMA_MemoryBurst_Single;
	DMA_InitStructure_I2C_RX.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
	DMA_Init(DMAx_StreamY_I2C_RX, &DMA_InitStructure_I2C_RX);

	// NVICの設定
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = DMAx_StreamY_IRQn_I2C_RX;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	DMA_ITConfig(DMAx_StreamY_I2C_RX, DMA_IT_TC, DISABLE); // 実際に使用するときに有効化する

	// 誤動作しないよう、DMA要求とDMAそのものを無効にする
	// これらはDMAを使用した送受信関数内で有効化される
	I2C_DMACmd(I2Cx_SIO, DISABLE);
	DMA_Cmd(DMAx_StreamY_I2C_TX, DISABLE);
	DMA_Cmd(DMAx_StreamY_I2C_RX, DISABLE);

	// I2C有効化
	I2C_Cmd(I2Cx_SIO, ENABLE);

	// タイムアウト監視用タイマ動作開始
	I2C_TIM_config_for_check_Timeout();
	reset_I2C_TIM();

	// I2Cペリフェラルが正常に動作するまで待機(無駄っぽいけどコメントアウト禁止)
	while( time_passing() < 0.01 ); //仕様上60ms程度が限度

	// 大事な基本設定なので、成功するまで繰り返す
	while(time_passing() >= TIMEOUT){
		// MPU9150AD変換開始(これを行ってから数msしないと、変換は開始されない)
		I2C_write_1byte(0         , MPU_6050_SLAVE_ADDRESS, 107);
		// コンパスとも通信できるようにする
		I2C_write_1byte(0b00000010, MPU_6050_SLAVE_ADDRESS, 55);
		// ジャイロ、加速度の測定レンジを±2000deg/s,±4gに設定
			//	I2C_DMA_write(TX_data, 3 , MPU_6050_SLAVE_ADDRESS); // バグの原因になるからつかいたくない
		I2C_write_1byte(0b00011000, MPU_6050_SLAVE_ADDRESS, 27);
		I2C_write_1byte(0b00001000, MPU_6050_SLAVE_ADDRESS, 28);
	}

	// MPU9150のAD変換値半自動取得の開始
	while(Start_I2C_DMA_for_communication_with_MPU9150() );

}

////////////////////////////////////////////////////////////////////////////////
/** @brief I2CとDMAを利用したMPU9150のAD変換値半自動取得の開始
           この関数実行後は割り込みを停止せずにはI2C_DMA_write、I2C_DMA_readを使えない
@param[in]  なし
@return なし
*///////////////////////////////////////////////////////////////////////////////
volatile uint8_t Start_I2C_DMA_for_communication_with_MPU9150(void)
{
	reset_I2C_TIM();
	// バスが空くまで待機
	while( (time_passing() < TIMEOUT) && (I2C_GetFlagStatus(I2Cx_SIO, I2C_FLAG_BUSY) ) );

	// DMACを一時的に停止
	DMA_Cmd(DMAx_StreamY_I2C_RX, DISABLE);
	while( (time_passing() < TIMEOUT) && (DMA_GetCmdStatus(DMAx_StreamY_I2C_RX) != DISABLE) );

	// スタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターライトモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (MPU_6050_SLAVE_ADDRESS)<<1, I2C_Direction_Transmitter);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED) ) );

	// 読みこみたいスレーブのアドレスを送信
	uint8_t register_address = 0;
	if(ADget_mode == 3) register_address = 59; // axel,temp,gyro
	else                register_address = 67; // gyro only
	I2C_SendData(I2Cx_SIO, register_address);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_BYTE_TRANSMITTED) ) );

	// リスタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターリードモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (MPU_6050_SLAVE_ADDRESS)<<1, I2C_Direction_Receiver);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED) ) );

	// DMAで最後のデータを受け取った時にNACKを返せるようにする
	I2C_DMALastTransferCmd(I2Cx_SIO, ENABLE);

	// 受信用DMAの設定
	uint32_t Data_num = 0;
	if(ADget_mode == 3) Data_num = 14; // axel,temp,gyro
	else                Data_num = 6; // gyro only
	DMA_InitStructure_I2C_RX.DMA_Memory0BaseAddr = (uint32_t)(&RX_data[6*ADget_mode]);
	DMA_InitStructure_I2C_RX.DMA_BufferSize = Data_num; // 転送回数
	DMA_Init(DMAx_StreamY_I2C_RX, &DMA_InitStructure_I2C_RX);
	// 割り込み有効化
	DMA_ITConfig(DMAx_StreamY_I2C_RX, DMA_IT_TC, ENABLE);
	// DMA要求を有効化
	I2C_DMACmd(I2Cx_SIO, ENABLE);
	// DMA有効化
	DMA_Cmd(DMAx_StreamY_I2C_RX, ENABLE);

// タイムアウトしたときは通信終了
	if( time_passing() > TIMEOUT ){
		// ストップコンディション生成
		I2C_GenerateSTOP(I2Cx_SIO, ENABLE);
		// DMA要求を無効化
		I2C_DMACmd(I2Cx_SIO, DISABLE);
		// ノーマルモードでは転送終了後DMAを一度無効にする
		DMA_Cmd(DMAx_StreamY_I2C_RX, DISABLE);

		// acknowledge有効化
		I2C_DMALastTransferCmd(I2Cx_SIO, DISABLE);
		I2C_AcknowledgeConfig(I2Cx_SIO, ENABLE);
		return 1; // 失敗を記録
	}
	return 0; // 正常にスタートした
}

////////////////////////////////////////////////////////////////////////////////
/** @brief I2CのDMA受信の完了割り込み関数を停止する
@details この割り込み関数より優先度の高い割り込み関数中で使うとどうなるか不明
@return なし
*///////////////////////////////////////////////////////////////////////////////
void Stop_I2C_DMA_for_communication_with_MPU9150(void)
{
	Stop_IT_flag = 1; // 通信再開停止要請
	while(Stop_IT_flag == 1);  // 通信終了を待機

	// 割り込み禁止
	DMA_ITConfig(DMAx_StreamY_I2C_RX, DMA_IT_TC, DISABLE);

	// 念のため、割り込みフラグのクリア
	DMA_ClearITPendingBit(DMAx_StreamY_I2C_RX, DMA_IT_TCIFx_RX);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief コンパスセンサの値を取得する
@details I2C_DMA転送完了割り込み関数より優先度の高い割り込み関数中で使うとどうなるか不明
         AK8975のレジスタはリトルエンディアンなので、int16_t型整数のポインタを直接入れてもよい
@return なし
*///////////////////////////////////////////////////////////////////////////////
void get_compass_AD_data(void* RX_Data_start_address)
{
	// 割り込み停止
	Stop_I2C_DMA_for_communication_with_MPU9150();

	// コンパスAD変換開始
	I2C_write_1byte(0b00000001,AK_8975_SLAVE_ADDRESS,10);

	// AD変換終了待機
	while(I2C_read_1byte(AK_8975_SLAVE_ADDRESS ,2) == 0 );

	// データ取得
	I2C_DMA_read(RX_Data_start_address, 6, AK_8975_SLAVE_ADDRESS, 3);

	// 割り込み開始
	while(Start_I2C_DMA_for_communication_with_MPU9150() );
}

////////////////////////////////////////////////////////////////////////////////
/** @brief I2CのDMA受信が完了した時によばれる割り込み関数
@details
関数名はマクロで変更すること
受信完了後に通信終了処理と再開処理を行うので、半自動でMPU9150のAD変換値を入手できる
@return なし
*///////////////////////////////////////////////////////////////////////////////
void I2C_DMAx_StreamY_IRQHandler() // Don't change the function name
{
	// 割り込みフラグのクリア
	DMA_ClearITPendingBit(DMAx_StreamY_I2C_RX, DMA_IT_TCIFx_RX);

	// 得られたAD変換値の取得
	uint8_t start_index = 6*ADget_mode;
	if(ADget_mode == 3){
		AD_data_axel_x =    (RX_data[start_index]<<8)    + RX_data[start_index+1];
		AD_data_axel_y =    (RX_data[start_index+2]<<8)  + RX_data[start_index+3];
		AD_data_axel_z =    (RX_data[start_index+4]<<8)  + RX_data[start_index+5];
		AD_data_temp =      (RX_data[start_index+6]<<8)  + RX_data[start_index+7];
		AD_data_gyro_x[3] = (RX_data[start_index+8]<<8)  + RX_data[start_index+9];
		AD_data_gyro_y[3] = (RX_data[start_index+10]<<8) + RX_data[start_index+11];
		AD_data_gyro_z[3] = (RX_data[start_index+12]<<8) + RX_data[start_index+13];
	}
	else{
		AD_data_gyro_x[ADget_mode] = (RX_data[start_index]<<8)   + RX_data[start_index+1];
		AD_data_gyro_y[ADget_mode] = (RX_data[start_index+2]<<8) + RX_data[start_index+3];
		AD_data_gyro_z[ADget_mode] = (RX_data[start_index+4]<<8) + RX_data[start_index+5];
	}

	// 作業用変数更新
	ADget_mode = (ADget_mode+1)%4;

// 通信終了処理

	// ストップコンディション生成
	I2C_GenerateSTOP(I2Cx_SIO, ENABLE);
	// DMA要求を無効化
	I2C_DMACmd(I2Cx_SIO, DISABLE);
	// ノーマルモードでは転送終了後DMAを一度無効にする
	DMA_Cmd(DMAx_StreamY_I2C_RX, DISABLE);

	// acknowledge有効化
	I2C_DMALastTransferCmd(I2Cx_SIO, DISABLE);
	I2C_AcknowledgeConfig(I2Cx_SIO, ENABLE);

// 通信再開処理

	// 割り込み停止要請がなければそのまま通信を再開する
	if(Stop_IT_flag == 0) while(Start_I2C_DMA_for_communication_with_MPU9150() );

	// 割り込み停止要請があれば再開しない、通信終了したことをフラグリセットで表す
	else Stop_IT_flag = 0;
}

////////////////////////////////////////////////////////////////////////////////
/** @brief I2Cを利用して、ある一つの対象スレーブに1バイト書き込む
@param[in]
			Data:送信するデータ
			slave_address:送信対象のスレーブアドレス
			register_address:送信対象のどのアドレスにデータを書き込むか指定
@return なし
*///////////////////////////////////////////////////////////////////////////////
void I2C_write_1byte(uint8_t Data, uint8_t slave_address , uint8_t register_address)
{
	reset_I2C_TIM();
	// バスが空くまで待機
	while( (time_passing() < TIMEOUT) && (I2C_GetFlagStatus(I2Cx_SIO, I2C_FLAG_BUSY) ) );

	// スタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターライトモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (slave_address)<<1, I2C_Direction_Transmitter);
	while( (time_passing() < TIMEOUT) &&  (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED) ) );

	// 書きこみたいスレーブのアドレスを送信
	I2C_SendData(I2Cx_SIO, register_address);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_BYTE_TRANSMITTED) ) );

	// データ送信
	I2C_SendData(I2Cx_SIO, Data);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_BYTE_TRANSMITTED) ) );

	// ストップコンディション生成
	I2C_GenerateSTOP(I2Cx_SIO, ENABLE);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief I2Cを利用して、ある一つの対象スレーブから1バイト読み込む
@param[in]
			slave_address:受信対象のスレーブアドレス
			register_address:受信対象のどのアドレスからデータを読み込むか指定
@return RxData:受信したデータ
*///////////////////////////////////////////////////////////////////////////////
uint8_t I2C_read_1byte(uint8_t slave_address ,uint8_t register_address)
{
	reset_I2C_TIM();
	uint8_t RxData;
	// バスが空くまで待機
	while( (time_passing() < TIMEOUT) && (I2C_GetFlagStatus(I2Cx_SIO, I2C_FLAG_BUSY) ) );

	// スタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターライトモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (slave_address)<<1, I2C_Direction_Transmitter);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED) ) );

	// 読みこみたいスレーブのアドレスを送信
	I2C_SendData(I2Cx_SIO, register_address);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_BYTE_TRANSMITTED) ) );

	// リスタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターリードモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (slave_address)<<1, I2C_Direction_Receiver);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED) ) );

	// 通信が終了した時にNACKが返せるようにする
	I2C_AcknowledgeConfig(I2Cx_SIO, DISABLE);

	// ストップコンディション生成
	I2C_GenerateSTOP(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_BYTE_RECEIVED) ) );

	RxData = I2C_ReceiveData(I2Cx_SIO);

	// acknowledge有効化
	I2C_AcknowledgeConfig(I2Cx_SIO, ENABLE);

	return RxData;
}

////////////////////////////////////////////////////////////////////////////////
/** @brief I2CとDMAを利用して、ある一つの対象スレーブに複数バイト書き込む
@param[in]
			Data_address:送信するデータが格納されているアドレス
			Data_num:送信するデータの個数
			slave_address:送信対象のスレーブアドレス
			送信対象にデータを書き込む起点のアドレスはTX_dataの一つ目に指定すること
@return なし
*///////////////////////////////////////////////////////////////////////////////
void I2C_DMA_write(uint8_t* Data_address, uint32_t Data_num ,uint8_t slave_address)
{
	reset_I2C_TIM();
	// バスが空くまで待機
	while( (time_passing() < TIMEOUT) && (I2C_GetFlagStatus(I2Cx_SIO, I2C_FLAG_BUSY) ) );

	// DMACを一時的に停止
	DMA_Cmd(DMAx_StreamY_I2C_TX, DISABLE);
	while( (time_passing() < TIMEOUT) && (DMA_GetCmdStatus(DMAx_StreamY_I2C_TX) != DISABLE) );

	// スタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターライトモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (slave_address)<<1, I2C_Direction_Transmitter);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED) ) );

	// 送信用DMAの設定
	DMA_InitStructure_I2C_TX.DMA_Memory0BaseAddr = (uint32_t)Data_address;
	DMA_InitStructure_I2C_TX.DMA_BufferSize = Data_num; // 転送回数
	DMA_Init(DMAx_StreamY_I2C_TX, &DMA_InitStructure_I2C_TX);
	// DMA要求を有効化
	I2C_DMACmd(I2Cx_SIO, ENABLE);
	// DMA有効化
	DMA_Cmd(DMAx_StreamY_I2C_TX, ENABLE);

	// 転送完了を待機
	while( (time_passing() < TIMEOUT) && (!DMA_GetFlagStatus(DMAx_StreamY_I2C_TX,DMA_FLAG_TCIFx_TX) ) );
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_BYTE_TRANSMITTED) ) );
	// ストップコンディション生成
	I2C_GenerateSTOP(I2Cx_SIO, ENABLE);
	// DMA要求を無効化
	I2C_DMACmd(I2Cx_SIO, DISABLE);
	// ノーマルモードでは転送終了後DMAを一度無効にする
	DMA_Cmd(DMAx_StreamY_I2C_TX, DISABLE);
	// 念のためフラグクリア
	DMA_ClearFlag(DMAx_StreamY_I2C_TX,DMA_FLAG_TCIFx_TX);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief I2CとDMAを利用して、ある一つの対象スレーブから複数バイト読み込む
@param[in]
			Data_address:読み込んだデータを格納する起点のアドレスを指定
			Data_num:データを読み込む個数
			slave_address:受信対象のスレーブアドレス
			register_address:受信対象のどのアドレスを起点としてデータを読み込むか指定
@return なし
*///////////////////////////////////////////////////////////////////////////////
void I2C_DMA_read(uint8_t* Data_address, uint32_t Data_num ,uint8_t slave_address ,uint8_t register_address)
{
	reset_I2C_TIM();
	// バスが空くまで待機
	while( (time_passing() < TIMEOUT) && (I2C_GetFlagStatus(I2Cx_SIO, I2C_FLAG_BUSY) ) );

	// DMACを一時的に停止
	DMA_Cmd(DMAx_StreamY_I2C_RX, DISABLE);
	while( (time_passing() < TIMEOUT) && (DMA_GetCmdStatus(DMAx_StreamY_I2C_RX) != DISABLE) );

	// スタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターライトモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (slave_address)<<1, I2C_Direction_Transmitter);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED) ) );

	// 読みこみたいスレーブのアドレスを送信
	I2C_SendData(I2Cx_SIO, register_address);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_BYTE_TRANSMITTED) ) );

	// リスタートコンディション生成
	I2C_GenerateSTART(I2Cx_SIO, ENABLE);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_MODE_SELECT) ) );

	// スレーブアドレス、マスターリードモード送信
	I2C_Send7bitAddress(I2Cx_SIO, (slave_address)<<1, I2C_Direction_Receiver);
	while( (time_passing() < TIMEOUT) && (!I2C_CheckEvent(I2Cx_SIO, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED) ) );

	// DMAで最後のデータを受け取った時にNACKを返せるようにする
	I2C_DMALastTransferCmd(I2Cx_SIO, ENABLE);

	// 受信用DMAの設定
	DMA_InitStructure_I2C_RX.DMA_Memory0BaseAddr = (uint32_t)Data_address;
	DMA_InitStructure_I2C_RX.DMA_BufferSize = Data_num; // 転送回数
	DMA_Init(DMAx_StreamY_I2C_RX, &DMA_InitStructure_I2C_RX);
	// DMA要求を有効化
	I2C_DMACmd(I2Cx_SIO, ENABLE);
	// DMA有効化
	DMA_Cmd(DMAx_StreamY_I2C_RX, ENABLE);

	// 転送完了を待機
	while( (time_passing() < TIMEOUT) && (!DMA_GetFlagStatus(DMAx_StreamY_I2C_RX,DMA_FLAG_TCIFx_RX) ) );
	// ストップコンディション生成
	I2C_GenerateSTOP(I2Cx_SIO, ENABLE);
	// DMA要求を無効化
	I2C_DMACmd(I2Cx_SIO, DISABLE);
	// ノーマルモードでは転送終了後DMAを一度無効にする
	DMA_Cmd(DMAx_StreamY_I2C_RX, DISABLE);

	// acknowledge有効化
	I2C_DMALastTransferCmd(I2Cx_SIO, DISABLE);
	I2C_AcknowledgeConfig(I2Cx_SIO, ENABLE);

	// 念のためフラグクリア
	DMA_ClearFlag(DMAx_StreamY_I2C_RX,DMA_FLAG_TCIFx_RX);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief タイムアウト監視タイマー初期化
@param[in] なし
@return なし
*///////////////////////////////////////////////////////////////////////////////
void I2C_TIM_config_for_check_Timeout(void)
{
	// タイマにクロック供給
	RCC_ClockCmd_I2C_TIM(ENABLE);

	// タイマのカウンタの設定値を計算する
	uint32_t clock;
	RCC_ClocksTypeDef RCC_Clocks;
	RCC_GetClocksFreq(&RCC_Clocks);
	if(RCC_Clocks.HCLK_Frequency == RCC_Clocks.PCLKx_Frequency_I2C_TIM){
		clock = RCC_Clocks.PCLKx_Frequency_I2C_TIM;
	}
	else{
		clock = RCC_Clocks.PCLKx_Frequency_I2C_TIM * 2;
	}

	// タイマの設定
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
	TIM_TimeBaseStructure.TIM_Prescaler =  (clock/1000000) -1; // 1MHzカウントアップ
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = 0XFFFF;
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseInit(TIMx_I2C_TIM, &TIM_TimeBaseStructure);

	// タイマ動作開始
	TIM_Cmd(TIMx_I2C_TIM, ENABLE);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief タイムアウト監視タイマーをリセット
@param[in] なし
@return なし
*///////////////////////////////////////////////////////////////////////////////
void reset_I2C_TIM(void)
{
	TIM_SetCounter(TIMx_I2C_TIM, 0);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief タイムアウト監視タイマーのリセットから経過した時間を返す
@param[in] なし
@return スタートから経過した時間[s]
*///////////////////////////////////////////////////////////////////////////////
volatile float time_passing(void)
{
	return (float)(TIM_GetCounter(TIMx_I2C_TIM) ) /1000000;
}
